@@ -1,12 +1,13 @@
 use color_eyre::*;
 
 use crate::{SCREENSHOTS_FOLDER, SHADER_DUMP_FOLDER, SHADER_PATH};
-use pilka_types::ImageDimentions;
+use pilka_types::{ImageDimentions, ShaderFlavor};
 
 use std::{
+    ffi::OsStr,
     fs::File,
     io::{self, BufWriter, Write},
-    path::Path,
+    path::{Path, PathBuf},
     time::{Duration, Instant},
 };
 
@@ -36,19 +37,19 @@ pub fn create_folder<P: AsRef<Path>>(name: P) -> io::Result<()> {
     Ok(())
 }
 
+#[derive(Debug)]
 pub struct Args {
     pub inner_size: Option<(u32, u32)>,
     pub record_time: Option<Duration>,
+    pub wgsl_mode: Option<()>,
 }
 
 pub fn parse_args() -> Args {
     let mut inner_size = None;
     let mut record_time = None;
-    for arg in std::env::args()
-        .skip(1)
-        .zip(std::env::args().skip(2))
-        .step_by(2)
-    {
+    let mut wgsl_mode = None;
+    let args = std::env::args().skip(1).collect::<Vec<_>>();
+    for arg in args.iter().zip(args.iter().cycle().skip(1)) {
         match arg.0.as_str() {
             "--record" => {
                 record_time =
@@ -64,6 +65,7 @@ pub fn parse_args() -> Args {
                     .map(|x| x.expect(&format!("Failed to parse window size: {}", arg.1)[..]));
                 inner_size = Some((iter.next().unwrap(), iter.next().unwrap()));
             }
+            "--wgsl" => wgsl_mode = Some(()),
             _ => {}
         }
     }
@@ -71,6 +73,7 @@ pub fn parse_args() -> Args {
     Args {
         record_time,
         inner_size,
+        wgsl_mode,
     }
 }
 
@@ -134,69 +137,57 @@ pub fn save_shaders<P: AsRef<Path>>(paths: &[P]) -> Result<()> {
     Ok(())
 }
 
-#[repr(C)]
-#[derive(Debug, Clone, Copy)]
-pub struct PushConstant {
-    pub pos: [f32; 3],
-    pub time: f32,
-    pub wh: [f32; 2],
-    pub mouse: [f32; 2],
-    pub mouse_pressed: u32,
-    pub frame: u32,
-    pub time_delta: f32,
-    pub record_period: f32,
+#[derive(Debug, Clone)]
+pub struct PilkaSpec {
+    pub frag: ShaderInfo,
+    pub comp: ShaderInfo,
+    pub vert: ShaderInfo,
+    pub glsl_prelude: Option<PathBuf>,
 }
 
-impl PushConstant {
-    pub fn as_slice(&self) -> &[u8] {
-        unsafe { any_as_u8_slice(self) }
-    }
-
-    pub fn size() -> u32 {
-        std::mem::size_of::<Self>() as _
-    }
+#[derive(Debug, Clone)]
+pub struct ShaderInfo {
+    pub path: PathBuf,
+    pub ty: ShaderFlavor,
 }
 
-/// # Safety
-/// Until you're using it on not ZST or DST it's fine
-pub unsafe fn any_as_u8_slice<T: Sized>(p: &T) -> &[u8] {
-    std::slice::from_raw_parts((p as *const T) as *const _, std::mem::size_of::<T>())
-}
+pub fn parse_folder(folder: &str) -> Result<PilkaSpec, Box<dyn std::error::Error>> {
+    let mut frag_si = None;
+    let mut comp_si = None;
+    let mut vert_si = None;
+    let mut prelude = None;
+    let shader_dir = PathBuf::new().join(folder);
+    for path in shader_dir.read_dir()? {
+        let path = path?.path();
 
-impl Default for PushConstant {
-    fn default() -> Self {
-        Self {
-            pos: [0.; 3],
-            time: 0.,
-            wh: [1920.0, 780.],
-            mouse: [0.; 2],
-            mouse_pressed: false as _,
-            frame: 0,
-            time_delta: 1. / 60.,
-            record_period: 10.,
+        let ty = match path.extension().and_then(OsStr::to_str) {
+            Some("wgsl") => ShaderFlavor::Wgsl,
+            Some("glsl" | "frag" | "comp" | "vert") => ShaderFlavor::Glsl,
+            _ => {
+                println!("This file have been ignored: {}", path.display());
+                continue;
+            }
+        };
+
+        let si = ShaderInfo { path, ty };
+        let file_name = si.path.to_str().unwrap();
+        if file_name.contains("frag") && frag_si.is_none() {
+            frag_si = Some(si);
+        } else if file_name.contains("comp") && comp_si.is_none() {
+            comp_si = Some(si);
+        } else if file_name.contains("vert") && vert_si.is_none() {
+            vert_si = Some(si);
+        } else if file_name.contains("prelude") {
+            prelude = Some(si.path);
+        } else {
+            println!("This file have been rejected: {}", si.path.display());
         }
     }
-}
 
-impl std::fmt::Display for PushConstant {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let time = Duration::from_secs_f32(self.time);
-        let time_delta = Duration::from_secs_f32(self.time_delta);
-        write!(
-            f,
-            "position:\t{:?}\n\
-             time:\t\t{:#.2?}\n\
-             time delta:\t{:#.3?}, fps: {:#.2?}\n\
-             width, height:\t{:?}\nmouse:\t\t{:.2?}\n\
-             frame:\t\t{}\nrecord_period:\t{}\n",
-            self.pos,
-            time,
-            time_delta,
-            1. / self.time_delta,
-            self.wh,
-            self.mouse,
-            self.frame,
-            self.record_period
-        )
-    }
+    Ok(PilkaSpec {
+        frag: frag_si.expect("Fragment shader is not provided"),
+        vert: vert_si.expect("Vertex shader is not provided"),
+        comp: comp_si.expect("Compute shader is not provided"),
+        glsl_prelude: prelude,
+    })
 }
